<?php
class actionAuthRegister extends cmsAction {

    public function run(){

        if (cmsUser::isLogged()) { $this->redirectToHome(); }

        $users_model = cmsCore::getModel('users');
        $form = $this->getForm('registration');

        //
        // Добавляем поле выбора группы,
        // при наличии публичных групп
        //
        $public_groups = $users_model->getPublicGroups();

        if ($public_groups) {

            $pb_items = array();
            foreach($public_groups as $pb) { $pb_items[ $pb['id'] ] = $pb['title']; }

            $form->addFieldToBeginning('basic',
                new fieldList('group_id', array(
                        'title' => LANG_USER_GROUP,
                        'items' => $pb_items
                    )
                )
            );

        }

        //
        // Добавляем в форму обязательные поля профилей
        //
        $content_model = cmsCore::getModel('content');
        $content_model->setTablePrefix('');
        $content_model->orderBy('ordering');
        $fields = $content_model->getRequiredContentFields('users');

        // Разбиваем поля по группам
        $fieldsets = cmsForm::mapFieldsToFieldsets($fields);

        // Добавляем поля в форму
        foreach($fieldsets as $fieldset){

            $fieldset_id = $form->addFieldset($fieldset['title']);

            foreach($fieldset['fields'] as $field){
                if ($field['is_system']) { continue; }
                $form->addField($fieldset_id, $field['handler']);
            }

        }

        $user = array();

        $is_submitted = $this->request->has('submit');

        if ($is_submitted){

            $user = $form->parse($this->request, $is_submitted);

            if (!isset($user['group_id'])) { $user['group_id'] = DEF_GROUP_ID; }

            // убираем поля которые не относятся к выбранной пользователем группе
            foreach($fieldsets as $fieldset){
                foreach($fieldset['fields'] as $field){

                    if (!$field['groups_edit']) { continue; }
                    if (in_array(0, $field['groups_edit'])) { continue; }

                    if (!in_array($user['group_id'], $field['groups_edit'])){
                        $form->disableField($field['name']);
                        unset($user[$field['name']]);
                    }

                }
            }

            $errors = $form->validate($this,  $user);

            if (!$errors){

                $result = $users_model->addUser($user);

                if ($result['success']){

                    cmsUser::addSessionMessage(LANG_REG_SUCCESS, 'success');

                    $back_url = cmsUser::sessionGet('auth_back_url') ?
                                cmsUser::sessionGet('auth_back_url', true) :
                                false;

                    if ($back_url){
                        $this->redirect($back_url);
                    } else {
                        $this->redirectToHome();
                    }

                } else {
                    $errors = $result['errors'];
                }

            }

            if ($errors){
                cmsUser::addSessionMessage(LANG_FORM_ERRORS, 'error');
            }

        }

        return cmsTemplate::getInstance()->render('reg', array(
            'user' => $user,
            'form' => $form,
            'errors' => isset($errors) ? $errors : false
        ));

    }

}