<?php

class comments extends cmsFrontend {

    protected $target_controller;
    protected $target_subject;

    public function __construct($request){

        parent::__construct($request);

        $this->target_controller = $this->request->get('target_controller');
        $this->target_subject = $this->request->get('target_subject');
        $this->target_id = $this->request->get('target_id');

    }

//============================================================================//
//============================================================================//

    public function getWidget(){

        $user = cmsUser::getInstance();

        $comments = $this->model->
                            lockFilters()->
                            filterEqual('target_controller', $this->target_controller)->
                            filterEqual('target_subject', $this->target_subject)->
                            filterEqual('target_id', $this->target_id)->
                            getComments();

        $is_tracking = $this->model->getTracking($user->id);

        $is_highlight_new = $this->request->hasInQuery('new_comments');

        if ($is_highlight_new && !$user->is_logged) { cmsCore::error404(); }

        $csrf_token_seed = implode('/', array($this->target_controller, $this->target_subject, $this->target_id));

        $template = cmsTemplate::getInstance();

        $template->setContext($this);

        $html = $template->render('list', array(
            'user' => $user,
            'target_controller' => $this->target_controller,
            'target_subject' => $this->target_subject,
            'target_id' => $this->target_id,
            'is_tracking' => $is_tracking,
            'is_highlight_new' => $is_highlight_new,
            'user' => $user,
            'comments' => $comments,
            'csrf_token_seed' => $csrf_token_seed,
        ));

        $template->restoreContext();

        return $html;

    }

//============================================================================//
//============================================================================//

    public function notifySubscribers($comment, $parent_comment=false){

        $subscribers = $this->model->
                                filterEqual('target_controller', $comment['target_controller'])->
                                filterEqual('target_subject', $comment['target_subject'])->
                                filterEqual('target_id', $comment['target_id'])->
                                getTrackingUsers();

        if (!$subscribers) { return; }

        // удаляем автора комментария из списка подписчиков
        $user_key = array_search($comment['user_id'], $subscribers);
        if ($user_key!==false) { unset($subscribers[$user_key]); }

        // удаляем автора родительского комментария из списка подписчиков,
        // поскольку он получит отдельное уведомление об ответе на комментарий
        if ($parent_comment){
            $parent_user_key = array_search($parent_comment['user_id'], $subscribers);
            if ($parent_user_key!==false) { unset($subscribers[$parent_user_key]); }
        }

        // проверяем что кто-либо остался в списке
        if (!$subscribers) { return; }

        $messenger = cmsCore::getController('messages');

        $messenger->addRecipients($subscribers);

        $messenger->sendNoticeEmail('comments_new', array(
            'page_url' => href_to_abs($comment['target_url']) . "#comment_{$comment['id']}",
            'page_title' => $comment['target_title'],
            'author_url' => href_to_abs('users', $comment['user_id']),
            'author_nickname' => $comment['user_nickname'],
            'comment' => $comment['content']
        ));

    }

    public function notifyParent($comment, $parent_comment){

        if ($comment['user_id'] == $parent_comment['user_id']) { return; }

        $messenger = cmsCore::getController('messages');

        $messenger->addRecipient($parent_comment['user_id']);

        $messenger->sendNoticeEmail('comments_reply', array(
            'page_url' => href_to_abs($comment['target_url']) . "#comment_{$comment['id']}",
            'page_title' => $comment['target_title'],
            'author_url' => href_to_abs('users', $comment['user_id']),
            'author_nickname' => $comment['user_nickname'],
            'comment' => $comment['content'],
            'original' => $parent_comment['content'],
        ));

    }

//============================================================================//
//============================================================================//

}
