<?php

class modelComments extends cmsModel{

//============================================================================//
//============================================================================//

    public function updateCommentContent($id, $content){

        return $this->update('comments', $id, array('content'=>$content));

    }

    public function updateCommentsPrivacy($is_private){

        return $this->updateFiltered('comments', array('is_private' => $is_private));

    }

//============================================================================//
//============================================================================//

    public function deleteComment($id){

        return $this->update('comments', $id, array('is_deleted'=>1));

    }

//============================================================================//
//============================================================================//

    public function addComment($comment){

        if ($comment['parent_id'] == 0){

            $comment['level'] = 1;
            $comment['ordering'] = $this->getNextThreadOrdering($comment['target_controller'], $comment['target_subject'], $comment['target_id']);

        } else {

            $parent_comment = $this->getComment($comment['parent_id']);

            $comment['level'] = $parent_comment['level'] + 1;

            $comment['ordering'] = $this->getNextParentOrdering($parent_comment);
            if (!$comment['ordering']) { $comment['ordering'] = $this->getNextThreadOrdering($comment['target_controller'], $comment['target_subject'], $comment['target_id']); }

            $this->incrementThreadOrdering($comment['target_controller'], $comment['target_subject'], $comment['target_id'], $comment['ordering']);

        }

        return $this->insert('comments', $comment);

    }

//============================================================================//
//============================================================================//

    public function getNextParentOrdering($parent_comment){

        $this->filterEqual('target_controller', $parent_comment['target_controller']);
        $this->filterEqual('target_subject', $parent_comment['target_subject']);
        $this->filterEqual('target_id', $parent_comment['target_id']);
        $this->filterLtEqual('level', $parent_comment['level']);
        $this->filterGt('ordering', $parent_comment['ordering']);
        $this->limit(1);

        return $this->getItem('comments', function($item){
            return $item['ordering'];
        });

    }

    public function getNextThreadOrdering($target_controller, $target_subject, $target_id){

        return $this->getMaxThreadOrdering($target_controller, $target_subject, $target_id) + 1;

    }

    public function getMaxThreadOrdering($target_controller, $target_subject, $target_id){

        $this->filterEqual('target_controller', $target_controller);
        $this->filterEqual('target_subject', $target_subject);
        $this->filterEqual('target_id', $target_id);

        return $this->getMaxOrdering('comments');

    }

    public function incrementThreadOrdering($target_controller, $target_subject, $target_id, $after){

        $this->filterEqual('target_controller', $target_controller);
        $this->filterEqual('target_subject', $target_subject);
        $this->filterEqual('target_id', $target_id);
        $this->filterGtEqual('ordering', $after);

        $this->increment('comments', 'ordering');

    }

//============================================================================//
//============================================================================//

    public function getCommentsCount(){

        $count = $this->getCount('comments');

        return $count;

    }

//============================================================================//
//============================================================================//

    public function getComments(){

        $this->select('u.nickname', 'user_nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->join('{users}', 'u', 'u.id = i.user_id');

        if (!$this->order_by){
            $this->orderBy('ordering');
        }

        return $this->get('comments', function($item, $model){

            $item['user'] = array(
                'id' => $item['user_id'],
                'nickname' => $item['user_nickname'],
                'avatar' => $item['user_avatar']
            );

            return $item;

        });

    }

//============================================================================//
//============================================================================//

    public function getComment($id){

        $this->select('u.nickname', 'user_nickname');
        $this->select('u.avatar', 'user_avatar');
        $this->join('{users}', 'u', 'u.id = i.user_id');

        return $this->getItemById('comments', $id, function($item, $model){

            $item['user'] = array(
                'id' => $item['user_id'],
                'nickname' => $item['user_nickname'],
                'avatar' => $item['user_avatar']
            );

            return $item;

        });

    }

//============================================================================//
//============================================================================//

    public function getTracking($user_id){

        $this->filterEqual('user_id', $user_id);

        return $this->getItem('comments_tracks');

    }

    public function addTracking($user_id, $target_controller, $target_subject, $target_id){

        // Получаем модель целевого контроллера
        $target_model = cmsCore::getModel( $target_controller );

        // Получаем URL и заголовок комментируемой страницы
        $target_info = $target_model->getTargetItemInfo($target_subject, $target_id);

        if (!$target_info){ return false; }

        return $this->insert('comments_tracks', array(
            'user_id' => $user_id,
            'target_controller' => $target_controller,
            'target_subject' => $target_subject,
            'target_id' => $target_id,
            'target_url' => $target_info['url'],
            'target_title' => $target_info['title'],
        ));

    }

    public function deleteTracking($id){

        return $this->delete('comments_tracks', $id);

    }

    public function toggleTracking($is_track, $user_id, $target_controller, $target_subject, $target_id){

        $track = $this->getTracking($user_id);

        if ($track && $is_track) { return true; }
        if (!$track && !$is_track) { return true; }

        if ($track && !$is_track) { return $this->deleteTracking($track['id']); }

        if (!$track && $is_track) { return $this->addTracking($user_id, $target_controller, $target_subject, $target_id); }

        return false;

    }

    public function getTrackingUsers(){

        return $this->get('comments_tracks', function($item, $model){
            return $item['user_id'];
        }, false);

    }

    public function getTrackedNewCounts($user_id, $date_after){

        $this->filterEqual('user_id', $user_id);

        $tracks = $this->get('comments_tracks');

        if (!$tracks) { return false; }

        $counts = array();
        $timestamp = strtotime($date_after);


        foreach($tracks as $track){

        $this->
            resetFilters()->
            filterTimestampGt('date_pub', $timestamp)->
            filterEqual('target_controller', $track['target_controller'])->
            filterEqual('target_subject', $track['target_subject'])->
            filterEqual('target_id', $track['target_id']);

            $count = $this->getCommentsCount();

            if ($count){

                $track['count'] = $count;

                $counts[] = $track;

            }

        }

        return $counts;

    }

//============================================================================//
//============================================================================//

}
