<?php

class actionContentItemAdd extends cmsAction {

    public function run(){

        $user = cmsUser::getInstance();

        // Получаем название типа контента
        $ctype_name = $this->request->get('ctype_name');

        // проверяем наличие доступа
        if (!cmsUser::isAllowed($ctype_name, 'add')) { cmsCore::error404(); }

        // Получаем тип контента
        $ctype = $this->model->getContentTypeByName($ctype_name);
        if (!$ctype) { cmsCore::error404(); }

        $groups_model = cmsCore::getModel('groups');
        $groups = $groups_model->getUserGroups($user->id);

        if (!$groups && $ctype['is_in_groups_only']){
            cmsUser::addSessionMessage(LANG_CONTENT_IS_IN_GROUPS_ONLY, 'error');
            $this->redirectBack();
        }

        // Получаем поля для данного типа контента
        $this->model->orderBy('ordering');
        $fields = $this->model->getContentFields($ctype_name);

        // Строим форму
        $form = new cmsForm();
        $fieldset_id = $form->addFieldset();

        // Если включены категории, добавляем в форму поле выбора категории
        if ($ctype['is_cats']){
            $fieldset_id = $form->addFieldset(LANG_CATEGORY);
            $form->addField($fieldset_id,
                new fieldList('category_id', array(
                        'generator' => function($item){
                            $content_model = cmsCore::getModel('content');
                            $tree = $content_model->getCategoriesTree($item['ctype_name']);
                            foreach($tree as $c){
                                $items[$c['id']] = str_repeat('- ', $c['ns_level']).' '.$c['title'];
                            }
                            return $items;
                        }
                    )
                )
            );
        }

        // Если этот контент можно создавать в группах (сообществах) то добавляем
        // поле выбора группы
        $groups_list = array();

        if ($ctype['is_in_groups'] || $ctype['is_in_groups_only']){

            $groups_list = ($ctype['is_in_groups_only']) ? array() : array('0'=>'');
            $groups_list = $groups_list + array_collection_to_list($groups, 'id', 'title');

            $fieldset_id = $form->addFieldset(LANG_GROUP);
            $form->addField($fieldset_id,
                new fieldList('parent_id', array(
                        'items' => $groups_list
                    )
                )
            );

        }

        // Разбиваем поля по группам
        $fieldsets = cmsForm::mapFieldsToFieldsets($fields, function($field, $user){

            // пропускаем системные поля
            if ($field['is_system']) { return false; }

            // проверяем что группа пользователя имеет доступ к редактированию этого поля
            if ($field['groups_edit'] && !$user->isInGroups($field['groups_edit'])) { return false; }

            return true;

        });

        // Добавляем поля в форму
        foreach($fieldsets as $fieldset){

            $fieldset_id = $form->addFieldset($fieldset['title']);

            foreach($fieldset['fields'] as $field){

                // добавляем поле в форму
                $form->addField($fieldset_id, $field['handler']);

            }

        }

        // Если ручной ввод SLUG, то добавляем поле для этого
        if (!$ctype['is_auto_url']){

            $fieldset_id = $form->addFieldset( LANG_SLUG );
            $form->addField($fieldset_id, new fieldString('slug', array(
                'prefix' => '/'.$ctype['name'].'/',
                'suffix' => '.html',
                'rules' => array( array('required') )
            )));

        }

        // Если разрешено управление видимостью, то добавляем поле
        if (cmsUser::isAllowed($ctype_name, 'privacy')) {

            $fieldset_id = $form->addFieldset( LANG_PRIVACY );
            $form->addField($fieldset_id, new fieldList('is_private', array(
                'items' => array(
                    0 => LANG_PRIVACY_PUBLIC,
                    1 => LANG_PRIVACY_PRIVATE,
                ),
                'rules' => array( array('number') )
            )));

        }

        //
        // Если ручной ввод ключевых слов или описания, то добавляем поля для этого
        //
        if (!$ctype['is_auto_keys'] || !$ctype['is_auto_keys']){
            $fieldset_id = $form->addFieldset( LANG_SEO );
            if (!$ctype['is_auto_keys']){
                $form->addField($fieldset_id, new fieldString('seo_keys', array(
                    'title' => LANG_SEO_KEYS,
                    'hint' => LANG_SEO_KEYS_HINT,
                    'rules' => array( array('max_length', 100) )
                )));
            }
            if (!$ctype['is_auto_desc']){
                $form->addField($fieldset_id, new fieldText('seo_desc', array(
                    'title' => LANG_SEO_DESC,
                    'hint' => LANG_SEO_DESC_HINT,
                )));
            }
        }

        $form = cmsEventsManager::hook("content_{$ctype['name']}_form", $form);

        // Форма отправлена?
        $is_submitted = $this->request->has('submit');

        // Парсим форму и получаем поля записи
        $item = $form->parse($this->request, $is_submitted);

        if ($is_submitted){

            // Проверям правильность заполнения
            $errors = $form->validate($this,  $item);

            if (!$errors){

                $item['parent_type'] = null;
                $item['parent_title'] = null;
                $item['parent_url'] = null;

                if (isset($item['parent_id'])){
                    if (array_key_exists($item['parent_id'], $groups_list) && $item['parent_id'] > 0){
                        $item['parent_type'] = 'group';
                        $item['parent_title'] = $groups_list[$item['parent_id']];
                        $item['parent_url'] = href_to_rel('groups', $item['parent_id'], array('content', $ctype_name));
                    } else {
                        $item['parent_id'] = null;
                    }
                }

                $item = cmsEventsManager::hook("content_{$ctype['name']}_before_add", $item);

                $item = $this->model->addContentItem($ctype, $item);

                cmsEventsManager::hook("content_{$ctype['name']}_after_add", $item);

                cmsCore::getController('activity')->addEntry($this->name, "add.{$ctype['name']}", array(
                    'subject_title' => $item['title'],
                    'subject_id' => $item['id'],
                    'subject_url' => href_to($ctype_name, $item['slug'] . '.html'),
                    'is_private' => isset($item['is_private']) ? $item['is_private'] : 0,
                    'group_id' => isset($item['parent_id']) ? $item['parent_id'] : null
                ));

                $this->redirectTo($ctype_name, $item['slug'] . '.html');

            }

            if ($errors){
                cmsUser::addSessionMessage(LANG_FORM_ERRORS, 'error');
            }

        }

        return cmsTemplate::getInstance()->render('item_form', array(
            'do' => 'add',
            'parent' => isset($parent) ? $parent : false,
            'ctype' => $ctype,
            'item' => $item,
            'form' => $form,
            'errors' => isset($errors) ? $errors : false
        ));

    }

}
