<?php

class actionContentItemView extends cmsAction {

    public function run(){

        $user = cmsUser::getInstance();

        // Получаем название типа контента и сам тип
        $ctype_name = $this->request->get('ctype_name');
        $ctype = $this->model->getContentTypeByName($ctype_name);
        if (!$ctype) { cmsCore::error404(); }
        if (!$ctype['options']['item_on']) { cmsCore::error404(); }

        // Получаем SLUG записи
        $slug = $this->request->get('slug');

        // Получаем запись
        $item = $this->model->getContentItemBySLUG($ctype_name, $slug);
        if (!$item) { cmsCore::error404(); }

        // Проверяем приватность
        if ($item['is_private']){
            if (!$user->isFriend($item['user_id'])){ cmsCore::error404(); }
        }

        // Проверяем ограничения доступа из других контроллеров
        if ($item['is_parent_hidden']){
            $is_parent_viewable = cmsEventsManager::hook("content_view_hidden", array('viewable'=>true, 'item'=>$item));
            if (!$is_parent_viewable){
                cmsUser::goLogin();
            }
        }

        $item['ctype_name'] = $ctype_name;

        if ($ctype['is_cats'] && $item['category_id'] > 1){
            $item['category'] = $this->model->getCategory($ctype_name, $item['category_id']);
        }

        // Получаем поля для данного типа контента
        $fields = $this->model->getContentFields($ctype_name);

        // Парсим значения полей
        foreach($fields as $name=>$field){
            $fields[ $name ]['html'] = $field['handler']->parse( $item[$name] );
        }

        // Рейтинг
        if ($ctype['is_rating']){

            $rating_controller = cmsCore::getController('rating', new cmsRequest(array(
                'target_controller' => $this->name,
                'target_subject' => $ctype['name']
            ), cmsRequest::CTX_INTERNAL));

            $is_rating_allowed = cmsUser::isAllowed($ctype['name'], 'rate');

            $item['rating_widget'] = $rating_controller->getWidget($item['id'], $item['rating'], $is_rating_allowed);

        }

        // Комментарии
        if ($ctype['is_comments']){

            $comments_controller = cmsCore::getController('comments', new cmsRequest(array(
                'target_controller' => $this->name,
                'target_subject' => $ctype['name'],
                'target_id' => $item['id']
            ), cmsRequest::CTX_INTERNAL));

            $item['comments_widget'] = $comments_controller->getWidget();

        }

        return cmsTemplate::getInstance()->render('item_view', array(
            'ctype' => $ctype,
            'fields' => $fields,
            'item' => $item,
            'user' => $user
        ));

    }

}
