<?php
class messages extends cmsFrontend {

    protected $useOptions = true;

    private $sender_id;
    private $recipients = array();
    private $is_ignore_options = false;

    /**
     * Устанавливает отправителя сообщения
     * @param int $user_id
     */
    public function setSender($user_id){

        $this->sender_id = $user_id;

    }

    /**
     * Добавляет получателя сообщения
     * @param int $user_id
     */
    public function addRecipient($user_id){

        $this->recipients[] = $user_id;

    }

    /**
     * Добавляет список получателей сообщения
     * @param array $list
     */
    public function addRecipients($list){

        $this->recipients = array_merge($this->recipients, $list);

    }

    public function clearRecipients(){
        $this->recipients = array();
    }

    /**
     * Отправляет сообщение
     * @param string $content
     * @return int | false
     */
    public function sendMessage($content){

        // Вырезаем теги кроме разрешенных
        $content = strip_tags($content, '<a><img><b><i><u><blockquote>');

        // Создаем контакты
        foreach($this->recipients as $contact_id){
            if (!$this->model->isContactExists($contact_id, $this->sender_id)){
                $this->model->addContact($contact_id, $this->sender_id);
            }
        }

        // Сохраняем сообщение
        $message_id = $this->model->addMessage($this->sender_id, $this->recipients, $content);

        // Обновляем даты последних сообщений в контактах
        if ($message_id){
            foreach($this->recipients as $contact_id){
                $this->model->updateContactsDateLastMsg($this->sender_id, $contact_id);
            }
        }

        return $message_id ? $message_id : false;

    }

    public function ignoreNotifyOptions(){
        $this->is_ignore_options = true;
        return $this;
    }

    /**
     * Отправляет уведомление через личные сообщения
     * @param string $notice_type
     * @param array $notice
     * @return int | false
     */
    public function sendNoticePM($notice_type, $notice){

        if (!$this->recipients){ return; }

        $options_only = $this->is_ignore_options ? false : array('pm', 'both');
        $recipients = cmsCore::getModel('users')->getNotifiedUsers($notice_type, $this->recipients, $options_only);
        if (!$recipients) { return false; }

        $this->is_ignore_options = false;

        return $this->model->addNotice($recipients, $notice);

    }

    /**
     * Отправляет email-уведомления указанного типа всем
     * подписанным пользователям
     * @param string $notice_type
     * @param string $notice Массив ключей и значений для замены в тексте письма
     * @return boolean
     */
    public function sendNoticeEmail($notice_type, $notice=array()){

        if (!$this->recipients){ return; }

        $letter_text = cmsCore::getLanguageTextFile("letters/{$notice_type}");
        if (!$letter_text){ return false; }

        $options_only = $this->is_ignore_options ? false : array('email', 'both');
        $recipients = cmsCore::getModel('users')->getNotifiedUsers($notice_type, $this->recipients, $options_only);
        if (!$recipients) { return false; }

        $this->is_ignore_options = false;

        $config = cmsConfig::getInstance();

        $notice = array_merge(array(
            'site' => $config->sitename,
            'date' => html_date(time()),
            'time' => html_time(time()),
        ), $notice);

        $letter_text = string_replace_keys_values($letter_text, $notice);

        $mailer = new cmsMailer();

        $success = true;

        foreach($recipients as $recipient){

            $letter_final_text = string_replace_keys_values($letter_text, $recipient);

            $letter_final_text = $mailer->parseSubject($letter_final_text);
            $letter_final_text = $mailer->parseAttachments($letter_final_text);

            $mailer->addTo($recipient['email'], $recipient['nickname']);

            $mailer->setBodyHTML( nl2br($letter_final_text) );

            $success = $success && $mailer->send();

            $mailer->clearTo()->clearAttachments();

        }

        return $success;

    }

}
