<?php

class rating extends cmsFrontend {

    private $target_controller;
    private $target_subject;

    public function __construct($request){

        parent::__construct($request);

        $this->target_controller = $this->request->get('target_controller');
        $this->target_subject = $this->request->get('target_subject');

    }

    public function getWidget($target_id, $current_rating, $is_enabled=true){

        $user = cmsUser::getInstance();

        // Этот пользователь уже голосовал?
        $is_voted = $user->is_logged ? $this->model->isUserVoted(array(
            'user_id' => $user->id,
            'target_controller' => $this->target_controller,
            'target_subject' => $this->target_subject,
            'target_id' => $target_id
        )) : false;

        $template = cmsTemplate::getInstance();

        $template->setContext($this);

        $html = $template->render('widget', array(
            'target_controller' => $this->target_controller,
            'target_subject' => $this->target_subject,
            'target_id' => $target_id,
            'is_enabled' => $is_enabled && !$is_voted,
            'is_voted' => $is_voted,
            'current_rating' => $current_rating ? $current_rating : 0,
            'user' => $user,
        ));

        $template->restoreContext();

        return $html;

    }

    public function actionVote(){

        if (!$this->request->isAjax()){ cmsCore::error404(); }

        // Получаем параметры
        $direction = $this->request->get('direction');
        $target_controller = $this->request->get('controller');
        $target_subject = $this->request->get('subject');
        $target_id = $this->request->get('id');

        $template = cmsTemplate::getInstance();

        // Проверяем валидность
        $is_valid = $this->validate_sysname($target_controller) &&
                    $this->validate_sysname($target_subject) &&
                    $this->validate_number($target_id) &&
                    in_array($direction, array('up', 'down'));

        if (!$is_valid){ $template->renderJSON(array('success' => false)); }

        $user = cmsUser::getInstance();

        // Объединяем всю информацию о голосе
        $vote = array(
            'user_id' => $user->id,
            'target_controller' => $target_controller,
            'target_subject' => $target_subject,
            'target_id' => $target_id,
            'score' => $direction=='up' ? 1 : -1
        );

        // Этот голос уже учитывался?
        $is_voted = $this->model->isUserVoted($vote);
        if ($is_voted){ $template->renderJSON(array('success' => false)); }

        // Добавляем голос в лог
        $this->model->addVote($vote);

        // Сообщаем модели целевого контента о том,
        // что нужно обновить агрегированный рейтинг и вернуть
        // нам итоговое значение
        $target_model = cmsCore::getModel( $target_controller );
        $rating = $target_model->getUpdatedRating($target_subject, $target_id, $vote);

        // Собираем результат
        $result = array(
            'success' => true,
            'rating' => html_signed_num( $rating ),
            'css_class' => html_signed_class( $rating ),
            'message' => LANG_RATING_VOTED
        );

        $template->renderJSON($result);

    }

}



