<?php

class users extends cmsFrontend {

    protected $useOptions = true;

    public function routeAction($action_name){

        if (!is_numeric($action_name)){ return $action_name; }

        $user_id = $action_name;

        $profile = $this->model->getUser($user_id);
        if (!$profile) { cmsCore::error404(); }

        $template = cmsTemplate::getInstance();
        $template->applyProfileStyle($profile);

        $core = cmsCore::getInstance();

        $this->current_params = $core->uri_params;
        $this->current_params[0] = $profile;

        if (!$core->uri_params){ return 'profile'; }

        $action_name = 'profile_' . $core->uri_params[0];

        return $action_name;

    }

    public function getProfileEditMenu($profile){

        $template = cmsTemplate::getInstance();

        $menu = array();

        $menu[] = array(
            'title' => LANG_USERS_EDIT_PROFILE_MAIN,
            'controller' => $this->name,
            'action' => $profile['id'],
            'params' => 'edit',
        );

        if ($template->hasProfileThemesOptions() && $this->options['is_themes_on']){
            $menu[] = array(
                'title' => LANG_USERS_EDIT_PROFILE_THEME,
                'controller' => $this->name,
                'action' => $profile['id'],
                'params' => array('edit', 'theme'),
            );
        }

        $menu[] = array(
            'title' => LANG_USERS_EDIT_PROFILE_NOTICES,
            'controller' => $this->name,
            'action' => $profile['id'],
            'params' => array('edit', 'notices'),
        );

        return $menu;

    }

    public function getProfileMenu($profile){

        $menu = array();

        $user = cmsUser::getInstance();

        $is_my_profile = ($user->id == $profile['id']);

        $menu[] = array(
            'title' => LANG_USERS_PROFILE_INDEX,
            'controller' => $this->name,
            'action' => $profile['id'],
        );

        $menu[] = array(
            'title' => $is_my_profile ? LANG_USERS_PROFILE_FEED : LANG_USERS_PROFILE_ACTIVITY,
            'controller' => $this->name,
            'action' => $profile['id'],
            'params' => 'activity',
        );

        if ($profile['content_count']){
            $menu[] = array(
                'title' => LANG_USERS_PROFILE_CONTENT,
                'controller' => $this->name,
                'action' => $profile['id'],
                'params' => array('content', $profile['first_ctype_name']),
                'url_mask' => href_to($this->name, $profile['id'], 'content'),
                'counter' => $profile['content_count']
            );
        }

        if ($profile['friends_count']){
            $menu[] = array(
                'title' => LANG_USERS_PROFILE_FRIENDS,
                'controller' => $this->name,
                'action' => $profile['id'],
                'params' => "friends",
                'counter' => $profile['friends_count']
            );
        }

        return $menu;

    }

    public function renderProfilesList($page_url, $dataset_name=false){

        if ($this->request->isInternal()){
            if ($this->useOptions){
               $this->options = $this->getOptions();
            }
        }

        $template = cmsTemplate::getInstance();
        $user = cmsUser::getInstance();

        $page = $this->request->get('page', 1);
        $perpage = 10;

        // Получаем поля
        $content_model = cmsCore::getModel('content');
        $content_model->setTablePrefix('');
        $content_model->orderBy('ordering');
        $fields = $content_model->getContentFields('users');

        $filters = array();

        // проверяем запросы фильтрации по полям
        foreach($fields as $name => $field){
            if (!$field['is_in_filter']) { continue; }
            if (!$this->request->has($name)){ continue; }
            $value = $this->request->get($name);
            if (!$value) { continue; }
            $this->model = $field['handler']->applyFilter($this->model, $value);
            $filters[$name] = $value;
        }

        // Постраничный вывод
        $this->model->limitPage($page, $perpage);

        // Получаем количество и список записей
        $total = $this->model->getUsersCount();
        $profiles = $this->model->getUsers();

        $template->setContext($this);

        $html = cmsTemplate::getInstance()->render('list', array(
            'page_url' => $page_url,
            'fields' => $fields,
            'filters' => $filters,
            'page' => $page,
            'perpage' => $perpage,
            'total' => $total,
            'profiles' => $profiles,
            'dataset_name' => $dataset_name,
            'user' => $user
        ), new cmsRequest(array(), cmsRequest::CTX_INTERNAL));

        $template->restoreContext();

        return $html;

    }

    public function getDatasets(){

        // Все (новые)
        $datasets['all'] = array(
                'name' => 'all',
                'title' => LANG_USERS_DS_LATEST,
                'order' => array('date_reg', 'desc')
        );

        // Онлайн
        if ($this->options['is_ds_online']){
            $datasets['online'] = array(
                'name' => 'online',
                'title' => LANG_USERS_DS_ONLINE,
                'order' => array('is_online', 'desc'),
                'filter' => function($model, $dset){
                    return $model->filterEqual('is_online', 1);
                }
            );
        }

        // Рейтинг
        if ($this->options['is_ds_rating']){
            $datasets['rating'] = array(
                'name' => 'rating',
                'title' => LANG_USERS_DS_RATED,
                'order' => array('karma desc, rating desc', '')
            );
        }

        // Популярные
        if ($this->options['is_ds_popular']){
            $datasets['popular'] = array(
                'name' => 'popular',
                'title' => LANG_USERS_DS_POPULAR,
                'order' => array('friends_count', 'desc')
            );
        }

        // Группы
        $f_groups = $this->model->getFilteredGroups();

        if ($f_groups){
            foreach($f_groups as $group){
                $datasets[ $group['name'] ] = array(
                    'name' => $group['name'],
                    'title' => $group['title'],
                    'order' => array('date_reg', 'desc'),
                    'filter' => function($model, $dset){
                        return $model->filterGroupByName($dset['name']);
                    }
                );
            }
        }

        return $datasets;

    }

}



