<?php

class modelUsers extends cmsModel{

//============================================================================//
//========================    ПОЛЬЗОВАТЕЛИ   =================================//
//============================================================================//

    public function getUsersCount(){

        return $this->getCount('{users}');

    }

//============================================================================//
//============================================================================//

    public function filterGroup($group_id){
        return $this->join('{users}_groups_members', 'm', "m.user_id = i.id AND m.group_id = '{$group_id}'");
    }

    public function filterGroups($groups_list){
        $groups_list = implode(',', $groups_list);
        return $this->join('{users}_groups_members', 'm', "m.user_id = i.id AND m.group_id IN ({$groups_list})");
    }

    public function filterGroupByName($group_name){
        $this->join('{users}_groups_members', 'm', "m.user_id = i.id");
        return $this->join('{users}_groups', 'g', "g.id = m.group_id AND g.name = '{$group_name}'");
    }

    public function getUsers(){

        $this->select("IFNULL(c.name, '')", 'city_name');
        $this->select("IFNULL(c.id, 0)", 'city_id');
        $this->joinLeft('geo_cities', 'c', 'c.id = i.city');

        return $this->get('{users}', function($user){

            $user['groups'] = cmsModel::yamlToArray($user['groups']);
            $user['theme'] = cmsModel::yamlToArray($user['theme']);

            $user['city'] = $user['city_id'] ? array(
                'id' => $user['city_id'],
                'name' => $user['city_name'],
            ) : false;

            return $user;

        });

    }

//============================================================================//
//============================================================================//

    public function getUser($id=false){

        $this->select("IFNULL(c.name, '')", 'city_name');
        $this->select("IFNULL(c.id, 0)", 'city_id');
        $this->joinLeft('geo_cities', 'c', 'c.id = i.city');

        if ($id){
            $user = $this->getItemById('{users}', $id);
        } else {
            $user = $this->getItem('{users}');
        }

        if (!$user) { return false; }

        $user['groups'] = cmsModel::yamlToArray($user['groups']);
        $user['theme'] = cmsModel::yamlToArray($user['theme']);
        $user['notify_options'] = cmsModel::yamlToArray($user['notify_options']);

        $user['city'] = $user['city_id'] ? array(
            'id' => $user['city_id'],
            'name' => $user['city_name'],
        ) : false;

        return $user;

    }

//============================================================================//
//============================================================================//

    public function addUser($user){

        $errors = false;

        if ($user['password1'] != $user['password2']){
            $errors['password1'] = LANG_REG_PASS_NOT_EQUAL;
            $errors['password2'] = LANG_REG_PASS_NOT_EQUAL;
            return array( 'success'=>false, 'errors'=>$errors );
        }

        $date_reg = date('Y-m-d H:i:s');
        $date_log = $date_reg;

        $password = $user['password1'];
        $password_salt = md5(implode(':', array($password, session_id(), time(), rand(0, 10000))));
        $password_salt = substr($password_salt, rand(1,8), 16);
        $password_hash = md5(md5($password) . $password_salt);

        $groups = !empty($user['groups']) ? $user['groups'] : array(DEF_GROUP_ID);

        if (isset($user['group_id'])) {
            $groups[] = $user['group_id'];
        }

        unset($user['password1']);
        unset($user['password2']);
        unset($user['group_id']);

        $user = array_merge($user, array(
            'groups' => $groups,
            'password' => $password_hash,
            'password_salt' => $password_salt,
            'date_reg' => $date_reg,
            'date_log' => $date_log,
        ));

        $id = $this->insert('{users}', $user);

        if ($id){

            $this->saveUserGroupsMembership($id, $groups);

            cmsCore::getController('activity')->addEntry('users', "signup", array(
                'user_id' => $id
            ));

        }

        return array(
            'success' => $id!==false,
            'errors' => false,
            'id' => $id
        );

    }

//============================================================================//
//============================================================================//

    public function updateUser($id, $user){

        $success    = false;
        $errors     = false;

        $email_exists_id = $this->db->getField('{users}', "email = '{$user['email']}'", 'id');

        if ($email_exists_id && ($email_exists_id != $id)){
            $errors['email'] = LANG_REG_EMAIL_EXISTS;
        }

        if (!empty($user['password1']) && !$errors){

            if (strlen($user['password1']) < 6) {
                $errors['password1'] = ERR_VALIDATE_MIN_LENGTH;
            }

            if ($user['password1'] != $user['password2']){
                $errors['password2'] = LANG_REG_PASS_NOT_EQUAL;
            }

            $password = $user['password1'];
            $password_salt = md5(implode(':', array($password, session_id(), time(), rand(0, 10000))));
            $password_salt = substr($password_salt, rand(1,8), 16);
            $password_hash = md5(md5($password) . $password_salt);

            $user['password'] = $password_hash;
            $user['password_salt'] = $password_salt;

        }

        if (!$errors){

            unset($user['password1']);
            unset($user['password2']);

            $user['groups'] = !empty($user['groups']) ? $user['groups'] : array(DEF_GROUP_ID);

            unset($user['city_id']);
            unset($user['city_name']);

            $success = $this->update('{users}', $id, $user);

            $this->saveUserGroupsMembership($id, $user['groups']);

        }

        return array(
            'success' => $success,
            'errors' => $errors,
            'id' => $id
        );

    }

//============================================================================//
//============================================================================//

    public function saveUserGroupsMembership($id, $groups){

        $this->delete('{users}_groups_members', $id, 'user_id');

        foreach($groups as $group_id){
            $this->insert('{users}_groups_members', array(
                'user_id' => $id,
                'group_id' => $group_id
            ));
        }

    }

//============================================================================//
//============================================================================//

    public function getUserContentCounts($id){

        $counts = array();

        $content_model = cmsCore::getModel('content');

        $ctypes = $content_model->getContentTypes();

        foreach($ctypes as $ctype){

            $content_model->filterEqual('user_id', $id);

            $count = $content_model->getContentItemsCount( $ctype['name'] );

            if ($count) {

                $counts[ $ctype['name'] ] = array(
                    'count' => $count,
                    'is_in_list' => $ctype['options']['list_on'],
                    'title' => $ctype['title']
                );

            }

        }

        return $counts;

    }

//============================================================================//
//=========================    УВЕДОМЛЕНИЯ   =================================//
//============================================================================//

    public function getUserNotifyOptions($id){

        return $this->getItemById('{users}', $id, function($item, $model){
            return cmsModel::yamlToArray($item['notify_options']);
        });

    }

    public function updateUserNotifyOptions($id, $options){

        return $this->update('{users}', $id, array('notify_options'=>$options));

    }

    public function getNotifiedUsers($notice_type, $id_list, $options_only=array()){

        $list = array();

        $this->filterIn('id', $id_list);

        $users = $this->get('{users}', function($user, $model){

            return array(
                'id' => $user['id'],
                'email' => $user['email'],
                'nickname' => $user['nickname'],
                'notify_options' => cmsModel::yamlToArray($user['notify_options'])
            );

        });

        if (!$users) { return false; }

        foreach($users as $user){

            if ($options_only){

                if (empty($user['notify_options'][$notice_type])){
                    $user['notify_options'][$notice_type] = 'email';
                }

                if (!in_array($user['notify_options'][$notice_type], $options_only)){
                    continue;
                }

            }

            unset($user['notify_options']);
            $list[] = $user;

        }

        return $list ? $list : false;

    }

//============================================================================//
//==============================    ГРУППЫ   =================================//
//============================================================================//

    public function getGroups($is_guests = false){

        if (!$is_guests) { $this->filterNotEqual('id', 1); }

        return $this->get('{users}_groups');

    }

    public function getPublicGroups(){

        return $this->filterNotEqual('id', 1)->
                        filterEqual('is_public', 1)->
                        get('{users}_groups');

    }

    public function getFilteredGroups(){

        return $this->filterNotEqual('id', 1)->
                        filterEqual('is_filter', 1)->
                        get('{users}_groups');

    }

    public function getGroup($id=false){

        return $this->getItemById('{users}_groups', $id);

    }

    public function updateGroup($id, $group){

        return $this->update('{users}_groups', $id, $group);

    }

    public function addGroup($group){

        return $this->insert('{users}_groups', $group);

    }

    public function deleteGroup($id){

        $this->join('{users}_groups_members', 'm', "m.user_id = i.id AND m.group_id = '{$id}'");

        $members = $this->getUsers();

        if ($members){

            foreach($members as $user){

                $groups = $user['groups'];

                // удаляем ID из массива групп пользователя
                // и переиндексируем ключи массива
                $groups = array_values( array_diff($groups, array($id)) );

                $this->update('{users}', $user['id'], array(
                    'groups' => $groups
                ));

            }

            $this->delete('{users}_groups_members', $id, "group_id");

        }

        $this->delete('{users}_groups', $id);

        return true;

    }

//============================================================================//
//==============================    ДРУЖБА   =================================//
//============================================================================//

    public function filterFriends($user_id){
        $user_id = intval($user_id);
        $this->joinInner('{users}_friends', 'f', "friend_id = i.id AND f.is_mutual = 1 AND f.user_id = '{$user_id}'");
        return $this;
    }

    public function getFriends($user_id){

        $this->select('u.id', 'id');
        $this->select('u.*');

        $this->joinInner('{users}', 'u', 'u.id = i.friend_id');

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('is_mutual', 1);

        if (!$this->order_by){
            $this->orderBy('u.date_log', 'desc');
        }

        return $this->get('{users}_friends');

    }


    public function getFriendsCount($user_id){

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('is_mutual', 1);

        $count = $this->getCount('{users}_friends');

        $this->resetFilters();

        return $count;
    }


    public function getFriendsIds($user_id){

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('is_mutual', 1);

        return $this->get('{users}_friends', function($item, $model){

            return $item['friend_id'];

        }, false);

    }

    public function isFriendshipRequested($user_id, $friend_id){

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('friend_id', $friend_id);
        $this->filterEqual('is_mutual', 0);

        $is_exists = (bool)$this->getCount('{users}_friends');

        $this->resetFilters();

        return $is_exists;

    }

    public function isFriendshipExists($user_id, $friend_id){

        $this->filterStart();
        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('friend_id', $friend_id);
        $this->filterEnd();

        $this->filterOr();

        $this->filterStart();
        $this->filterEqual('user_id', $friend_id);
        $this->filterEqual('friend_id', $user_id);
        $this->filterEnd();

        $is_exists = (bool)$this->getCount('{users}_friends');

        $this->resetFilters();

        return $is_exists;

    }


    public function isFriendshipMutual($user_id, $friend_id){

        $this->filterStart();
            $this->filterStart();
            $this->filterEqual('user_id', $user_id);
            $this->filterEqual('friend_id', $friend_id);
            $this->filterEnd();

            $this->filterOr();

            $this->filterStart();
            $this->filterEqual('user_id', $friend_id);
            $this->filterEqual('friend_id', $user_id);
            $this->filterEnd();
        $this->filterEnd();

        $this->filterAnd();

        $this->filterEqual('is_mutual', 1);

        $is_exists = (bool)$this->getCount('{users}_friends');

        $this->resetFilters();

        return $is_exists;

    }


    public function addFriendship($user_id, $friend_id){

        $is_mutual = false;

        if ($this->isFriendshipRequested($friend_id, $user_id)){

            $this->filterEqual('user_id', $friend_id);
            $this->filterEqual('friend_id', $user_id);

            $this->updateFiltered('{users}_friends', array(
                'is_mutual' => true
            ));

            $is_mutual = true;

        }

        if ($is_mutual){

            $this->filterEqual('id', $user_id)->increment('{users}', 'friends_count');
            $this->filterEqual('id', $friend_id)->increment('{users}', 'friends_count');

            $friend = $this->getUser($friend_id);

            cmsCore::getController('activity')->addEntry('users', "friendship", array(
                'subject_title' => $friend['nickname'],
                'subject_id' => $friend_id,
                'subject_url' => href_to('users', $friend_id),
            ));

        }

        return $this->insert('{users}_friends', array(
            'user_id' => $user_id,
            'friend_id' => $friend_id,
            'is_mutual' => $is_mutual
        ));

    }

    public function deleteFriendship($user_id, $friend_id){

        if ($this->isFriendshipMutual($user_id, $friend_id)){
            $this->filterEqual('id', $user_id)->decrement('{users}', 'friends_count');
            $this->filterEqual('id', $friend_id)->decrement('{users}', 'friends_count');
        }

        $this->filterEqual('user_id', $user_id);
        $this->filterEqual('friend_id', $friend_id);
        $this->deleteFiltered('{users}_friends');

        $this->filterEqual('user_id', $friend_id);
        $this->filterEqual('friend_id', $user_id);
        $this->deleteFiltered('{users}_friends');

    }

//============================================================================//
//============================================================================//

}
