var grid_options = {}
var is_loading = true;
var selected_rows = [4,7,12];

function setDataGridOptions(options){
    
    grid_options = options;
    
}

function initDataGrid(){

    if (grid_options.is_sortable){
        $('.datagrid th.sortable').click(function(){
            clickGridHeader($(this).attr('rel'));
        });
    }

    if (grid_options.is_filter){
        $('.datagrid .filter .input').keypress(function(event){

            if (event.which == 13) {

                event.preventDefault();

                $('.datagrid .filter .input').each(function(){
                    var filter = $(this).attr('rel');
                    $('#datagrid_filter input[name='+filter+']').val($(this).val());
                });

                grid_options.page = 1;

                setPage();
                loadGridRows();

            }

        });
    }

    setOrdering();
    loadGridRows();

}

function dataGridSubmit(url){

    if (selected_rows.length==0 && !grid_options.is_draggable) { return false; }

    $('#datagrid_form').html('');
    $('#datagrid_form').attr('action', url);

    if (selected_rows.length > 0){

        $.each(selected_rows, function(){
            var row_id = $(this).attr('id');
            $('#datagrid_form').append('<input type="hidden" name="selected[]" value="'+this+'" />');
        })

    }

    if (grid_options.is_draggable){

        $('.datagrid tbody tr').each(function(){
            var row_id = $(this).attr('id');
            $('#datagrid_form').append('<input type="hidden" name="items[]" value="'+row_id+'" />');
        })
    }

    $('#datagrid_form').submit();

    return true;

}

function clickGridHeader(name){

    if (grid_options.order_by != name){
        grid_options.order_to = 'asc';
    } else {
        grid_options.order_to = grid_options.order_to == 'desc' ? 'asc' : 'desc';
    }

    grid_options.order_by = name;

    setOrdering();

    loadGridRows();

}

function setOrdering(){

    if (!grid_options.is_sortable) { return false; }
    
    $('#datagrid_filter input[name=order_by]').val(grid_options.order_by);
    $('#datagrid_filter input[name=order_to]').val(grid_options.order_to);

    $('.datagrid th').removeClass('sort_asc').removeClass('sort_desc');
    $('.datagrid th[rel='+grid_options.order_by+']').addClass('sort_'+grid_options.order_to);

}

function setPage(){

    if (!grid_options.is_pagination) { return false; }

    $('#datagrid_filter input[name=page]').val(grid_options.page);

}

function loadGridRows(){

    is_loading = true;

    setTimeout(showLoadIndicator, 500);

    var filter_query = $('#datagrid_filter').serialize();

    $.post(grid_options.url, {filter: filter_query}, gridRowsLoaded, 'json');

}

function gridRowsLoaded(result){

    is_loading = false;

    hideLoadIndicator();

    $('.datagrid tbody tr[class!=filter]').remove();

    if(!result.rows.length){
        var columns_count = $('.datagrid thead th').length;
        $('.datagrid tbody').append('<tr><td colspan="'+columns_count+'">'+LANG_LIST_EMPTY+'</td></tr>');
        return;
    }

    $.each(result.rows, function(){
       var row = this;
       var row_html = '<tr id="'+row[0]+'">';
       $.each(row, function(index){
           if (index>0 || grid_options.show_id) {
                row_html = row_html + '<td>' + this + '</td>';
           }
       });
       row_html = row_html + '</tr>';
       $('.datagrid tbody').append(row_html);
    });

    $('.datagrid tbody tr:odd').addClass('odd');

    if (grid_options.is_draggable) {
        $('#datagrid').tableDnD({
            onDragClass: 'dragged',
            onDrop: function(table, row) {
                $('.datagrid tbody tr').removeClass('odd');
                $('.datagrid tbody tr:odd').addClass('odd');
            }
        });
    }

    if (grid_options.is_pagination && result.pages_count > 0) {
        if (result.pages_count != grid_options.pages_count){

            grid_options.pages_count = result.pages_count;

            $(".datagrid_pagination").paginate({
                        count 		: result.pages_count,
                        start 		: 1,
                        display     : 7,
                        border					: false,
                        images					: true,
                        mouse					: 'press',
                        border_color  			: '#fff',
                        text_color  			: '#333',
                        background_color    	: '#fff',
                        border_hover_color		: '#7d929d',
                        text_hover_color  		: '#fff',
                        background_hover_color	: '#7d929d',
                        onChange     			: function(page){
                                                        grid_options.page = page;
                                                        setPage();
                                                        loadGridRows();
                                                  }
            });

        }
    }

}

function showLoadIndicator(){

    if (!is_loading) {return;}

    $('.datagrid_loading').show();

    var pos = $('.datagrid tbody').offset();
    var w = $('.datagrid tbody').width();
    var h = $('.datagrid tbody').height();

    var iw = $('.datagrid_loading .indicator').width();
    var ih = $('.datagrid_loading .indicator').height();

    var itop = h/2 - ih/2 - 4;
    var ileft = w/2 - iw/2 - 4;

    $('.datagrid_loading').css('left', pos.left+'px').css('top', pos.top+'px');
    $('.datagrid_loading').css('width', w+'px').css('height', h+'px').css('line-height', h+'px');

    $('.datagrid_loading .indicator').css('top', itop+'px').css('left', ileft+'px');

}

function hideLoadIndicator(){

    $('.datagrid_loading').hide();

}